@@ -35,8 +35,7 @@ class Dgram(object):
 
     """
 
-    def __init__(self, pollManager, peerName, bindAddr=None, peerAddr=None,
-                 onReceive=None):
+    def __init__(self, pollManager, peerName, bindAddr=None, peerAddr=None, onReceive=None):
         """Constructor:
 
         :Parameters:
@@ -98,7 +97,7 @@ class Dgram(object):
         This simply sends the the string `bytes` to the peer.
 
         :Param bytes:
-            The string containing the bytes to send.
+            The byte string containing the bytes to send.
         :Param count:
             If supplied and non-zero then only the first `count` bytes will be
             sent.
@@ -205,9 +204,8 @@ class Connecter(object):
     Currently only AF_INET address family sockets are supported.
     """
 
-    def __init__(self, pollManager, peerName, peerAddr, connectTimes,
-                 onConnect=None, bindAddress=None, maxErrorRetries=5,
-                 onError=None):
+    def __init__(self, pollManager, peerName, peerAddr, connectTimes, onConnect=None, bindAddress=None,
+                 maxErrorRetries=5, onError=None):
         """Constructor:
 
         :param pollManager:
@@ -230,7 +228,7 @@ class Connecter(object):
             The address to bind to
         :param maxErrorRetries:
             The maximum number of times we will try to connect to a socket when
-            errors occur
+            errors occur - this includes attempts to ssl handshake
         :Param onError:
             Callback function to call on error
         """
@@ -246,6 +244,7 @@ class Connecter(object):
         delay, self.retryPeriod = connectTimes
         self.maxErrorRetries = maxErrorRetries
         self.errorsOnConnection = 0
+        self.sslHandshakeErrorsOnConnection = 0
         self.s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
         self.s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
         self.s.setblocking(0)
@@ -256,6 +255,8 @@ class Connecter(object):
         self.tid = self.pollManager.addRepeatingTimeout(
                 self.retryPeriod, self.onTryconnect)
         self._fileno = self.s.fileno()
+        self._connected = False
+        self._handshakeComplete = False
 
     def isSslConnection(self):
         return bool(self._sslHandshaker is not None)
@@ -265,56 +266,100 @@ class Connecter(object):
         self.s = ssl.wrap_socket(self.s, *args, **kwargs)
         self._useSSL = True
 
-    def onTryconnect(self):
-        """Invoked by the `PollManager` until a connection has been established
-        or self.maxErrorRetries has been reached at which point we abandon
-        trying to connect
+    def _connectionFailed(self):
+        error_count = self.sslHandshakeErrorsOnConnection + self.errorsOnConnection
 
-        :return None
-        """
-        connected = False
+        log.error(
+            "%d errors have occurred when trying to connect,"
+            " abandoning connection attempts",
+            error_count
+        )
+        if self.error_callback is not None:
+            self.error_callback()
+
+        self.shutDown()
+
+    def _setupConnection(self):
         try:
             self.s.connect(self.peerAddr)
-            connected = True
+            self._connected = True
         except socket.error as exc:
             code = exc.args[0]
-            if code == errno.EINPROGRESS:
-                return
-            else:
+            # EINPROGRESS is expected and does not count towards the errors
+            if code != errno.EINPROGRESS:
                 self.errorsOnConnection += 1
                 log.warning(
-                    "An unhandled exception occured when trying to connect"
-                    " to %s:%s", self.peerName, self.peerAddr)
+                    "An exception occurred when trying to connect to %s:%s\nException was %s",
+                    self.peerName,
+                    self.peerAddr,
+                    str(exc)
+                )
+                log.warning(
+                    "Errors on conn %d, max errors %d",
+                    self.errorsOnConnection,
+                    self.maxErrorRetries
+                )
 
-                if self.errorsOnConnection > self.maxErrorRetries:
-                    log.error(
-                        "%d errors have occurred when trying to connect,"
-                        " abandoning connection attempts",
-                        self.errorsOnConnection)
-                    if self.error_callback is not None:
-                        self.error_callback()
-                else:
-                    # Keep trying for now so just return here
-                    return
-
-        # Remove the timer here, if we are not yet connected this will stop any
-        # more attempts to do so - if we are there is no need to keep trying
-        # to connect
-        self.pollManager.removeTimeout(self.tid)
-        self.tid = None
-
-        if not connected:
-            self.shutDown()
-            return
-
-        if self._useSSL:
-            # Do the handshaking while the socket is in blocking mode. In
-            # theory you can do it in non-blocking mode, but I think there
-            # is a nasty timing bug in Python's wrapper or some versions of
-            # the SSL library itself.
+    def _setupSSLHandshake(self):
+        # Do the handshaking while the socket is in blocking mode. In
+        # theory you can do it in non-blocking mode, but I think there
+        # is a nasty timing bug in Python's wrapper or some versions of
+        # the SSL library itself.
+        try:
             self.s.setblocking(1)
             self.s.do_handshake()
             self.s.setblocking(0)
+            self._handshakeComplete = True
+        except ssl.SSLError as exc:
+            self.sslHandshakeErrorsOnConnection += 1
+            log.warning("SSL exception performing SSL handshake.\nException was %s", str(exc))
+            log.warning(
+                "Errors on conn %d, SSL handshake errors %d max errors %d",
+                self.errorsOnConnection,
+                self.sslHandshakeErrorsOnConnection,
+                self.maxErrorRetries
+            )
+
+    def onTryconnect(self):
+        """Invoked by the `PollManager` until a connection has been established
+        or self.maxErrorRetries has been reached at which point we abandon
+        trying to connect.
+
+        If SSL is in use this function will continue to be invoked after we are connected until the
+        SSL handshake completes or self.maxErrorRetries has been reached.
+
+        :return None
+        """
+        if not self._connected:
+            self._setupConnection()
+
+            if not self._connected:
+                # If we're still not connected check if we should give up and return
+                if self.errorsOnConnection > self.maxErrorRetries:
+                    # This will also remove the pollmanger timer so this function is no longer
+                    # called
+                    self._connectionFailed()
+
+                return
+
+        # We have a connection - do we need to do the ssl handshake
+        if self._useSSL and not self._handshakeComplete:
+            self._setupSSLHandshake()
+
+            if not self._handshakeComplete:
+                # Check if we should give up and return
+                error_count = self.errorsOnConnection + self.sslHandshakeErrorsOnConnection
+                if error_count > self.maxErrorRetries:
+                    # This will also remove the pollmanger timer so this function is no longer
+                    # called
+                    self._connectionFailed()
+
+                return
+
+        # If we reach this point we're connected and have performed the handshake if required so
+        # remove the timer
+        self.pollManager.removeTimeout(self.tid)
+        self.tid = None
 
         active, ret = self.onConnectCB(self.peerName, self)
         if not active:
@@ -327,6 +372,7 @@ class Connecter(object):
         """
         if self.tid is not None:
             self.pollManager.removeTimeout(self.tid)
+            self.tid = None
         self.s.close()
 
 
@@ -383,8 +429,8 @@ class SocketConn(object):
         self.onReceiveCB = CallbackRef(onReceive)
         self.onCloseCB = CallbackRef(onClose)
         self.onErrorCB = CallbackRef(onError)
-        self.inBuf = ""
-        self.outBuf = ""
+        self.inBuf = six.b("")
+        self.outBuf = six.b("")
         self.pollManager.addInputCallback(s, self._onInputActivity,)
         self._eofSeen = False
         self._native = native
@@ -404,7 +450,7 @@ class SocketConn(object):
         """Handle attempt to read from socket when EOF has occurred.
 
         This is invoked by the `._read` method when EOF has been seen and
-        there is no bufferred input data. It always invokes the onCloseCB and
+        there is no buffered input data. It always invokes the onCloseCB and
         the first time it is called will remove callbacks and cleanly shutdown
         the socket.
         """
@@ -416,7 +462,7 @@ class SocketConn(object):
                 self.s.shutdown(socket.SHUT_RDWR)
                 self.s.close()
             except socket.error:
-                pass # We are really beyond caring!
+                pass  # We are really beyond caring!
             self.s = None
 
     def _read(self):
@@ -441,6 +487,10 @@ class SocketConn(object):
             except ssl.SSLError as exc:
                 if exc.args[0] == ssl.SSLWantReadError:
                     break
+                # Need to break on any ssl error, otherwise we end up in a busy loop.
+                log.warning("SSL exception on read: %s", str(exc))
+
+                break
             except socket.error as exc:
                 if exc.args[0] == errno.ECONNRESET:
                     got_eof = True
@@ -451,7 +501,7 @@ class SocketConn(object):
         if data:
             # We read some data, so update the input buffer. If we also got EOF
             # then that will be handled when this gets called next time around.
-            self.inBuf += ''.join(data)
+            self.inBuf += six.b("").join(data)
 
         elif got_eof:
             # The socket has been closed and we have no buffered data.
@@ -472,21 +522,21 @@ class SocketConn(object):
         if self.inBuf:
             active, ret = self.onReceiveCB(self)
             if not active:
-                self.inBuf = ""
+                self.inBuf = six.b("")
 
     def _doInitialRead(self, fd):
         self._read()
         if self.inBuf:
             active, ret = self.onReceiveCB(self)
             if not active:
-                self.inBuf = ""
+                self.inBuf = six.b("")
 
     def read(self):
         """Read all the data in the input buffer.
 
         The buffer is cleared.
         """
-        s, self.inBuf = self.inBuf, ""
+        s, self.inBuf = self.inBuf, six.b("")
         return s
 
     def peek(self):
@@ -497,6 +547,15 @@ class SocketConn(object):
         """Read a complete message from the input buffer.
 
         The read bytes are removed from the input buffer.
+
+        Note this can ONLY be used if the messages have headers, if they do not
+        use the read function.
+
+        +---+---+---+---+---+--- ... ---+---+
+        |    Length     |      Payload      |
+        +---+---+---+---+---+--- ... ---+---+
+
+        |<-------------- Length ----------->|
         """
         if len(self.inBuf) < 4:
             return None
@@ -582,7 +641,7 @@ class SocketConn(object):
                 self.pollManager.addTimeout(0.0, self.onCloseCB, self)
                 self.pollManager.removeOutputCallback(self._fileno)
                 self.pollManager.removeInputCallback(self._fileno)
-                self.outBuf = ""
+                self.outBuf = six.b("")
                 try:
                     self.s.shutdown(socket.SHUT_RDWR)
                     self.s.close()
@@ -622,9 +681,8 @@ class PipeConn(object):
     for mock objects.
     """
 
-    def __init__(self, isWrite, pipePath, peerName, pollManager,
-                 onReceive=None, onClose=None, onError=None, native=False,
-                 addMsgLengthOnSend=False):
+    def __init__(self, isWrite, pipePath, peerName, pollManager, onReceive=None, onClose=None, onError=None,
+                 native=False, addMsgLengthOnSend=False):
         """Constructor:
 
         :Parameters:
@@ -765,14 +823,14 @@ class PipeConn(object):
         self.inBuf += s
         active, ret = self.onReceiveCB(self)
         if not active:
-            self.inBuf = ""
+            self.inBuf = six.b("")
 
     def read(self):
         """Read all the data in the input buffer.
 
         The buffer is cleared.
         """
-        s, self.inBuf = self.inBuf, ""
+        s, self.inBuf = self.inBuf, six.b("")
         return s
 
     def peek(self):
@@ -791,7 +849,7 @@ class PipeConn(object):
         |    Length     |      Payload      |
         +---+---+---+---+---+--- ... ---+---+
 
-        |<-------------- Length ----------->|
+                        |<---- Length ----->|
         """
         if len(self.inBuf) < 4:
             return None
@@ -849,7 +907,7 @@ class PipeConn(object):
                     self.pipeF.close()
                 except IOError:
                     pass  # We are really beyond caring!
-                self.s = None
+
                 return
 
         # Try poking things. so we detect pipe closure earlier rather than
@@ -873,7 +931,7 @@ class PipeConn(object):
             # If the connection has closed, ...
             self.pollManager.removeOutputCallback(self._fileno)
             self.pollManager.removeInputCallback(self._fileno)
-            self.outBuf = ""
+            self.outBuf = six.b("")
             try:
                 self.pipeF.close()
             except (IOError, OSError):
