@@ -25,7 +25,6 @@ from CleverSheep.Prog import Files
 from CleverSheep.Test.Tester import Core
 from CleverSheep.Test.Tester import Context
 from CleverSheep.Test.Tester import Assertions
-from CleverSheep.Test.Tester import WatchDog
 from CleverSheep.Test.Tester import Errors
 from CleverSheep.Test.Tester import Suites
 from CleverSheep.Test.Tester import options
@@ -261,21 +260,16 @@ class Manager(object):
     def _execute(self, reportMan, env):
         """Execute all selected tests in order.
 
-        TODO: This is not necessarily execution. A **much** better name is
-        required.
-
         :Parameters:
           reportMan
             The test reporting manager instance.
           env
             The test execution environment.
-
         """
         loadInfo()
         reportMan.setMode("EXECUTION")
 
         all_passed = True
-        status = Coordinator.getServiceProvider("status")
         for prevTest, test, nextTest in self.selectedTests():
             current_test_passed = True
 
@@ -337,7 +331,6 @@ class Manager(object):
         status.setField("commentary", "")
         reportMan = Coordinator.getServiceProvider("report_manager")
         env = TestExecEnvironment(self._collection)
-        logging = Coordinator.getServiceProvider("logging")
         tty = Coordinator.getServiceProvider("tty")
         tty.stdout.setLog(reportMan.debug)
         tty.stderr.setLog(reportMan.error)
@@ -356,10 +349,6 @@ class Manager(object):
             self.context = None
             self.mustStop = False
             reportMan.stopTestRun()
-            # TODO: This code breaks something.
-            if 0:
-                tty.stdout.setLog(None)
-                tty.stderr.setLog(None)
 
         # TODO: Is this a sensible thing to return?
         self.exitCode |= env.exitCode
@@ -462,6 +451,7 @@ Coordinator.registerProvider("cscore", ("manager",), Manager())
 #       abstraction or part of the API for manager writers.
 class _TestProcessor(object):
     _doDump = False
+
     def __init__(self, collection):
         null_test = Core.NullTest()
         self.reportMan = Coordinator.getServiceProvider("report_manager")
@@ -556,7 +546,7 @@ class _TestProcessor(object):
 
     def _execute(self, test, level):
         status = Coordinator.getServiceProvider("status")
-        runnable = test is None or not test.hasFailingAncestor()
+        runnable = test is not None
         if runnable:
             self.runner = None
             runnable = self._doSetUp()
@@ -632,19 +622,8 @@ class _TestDetailer(_TestProcessor):
             self.reportMan.enterSuite(suite)
 
 
-def addHang(lines, hangText):
-    res = []
-    pad = " " * len(hangText)
-    for i, line in enumerate(lines):
-        if i:
-            res.append(pad + line)
-        else:
-            res.append(hangText + line)
-    return res
-
-
 class SphinxBase(object):
-    """Base class for SPhinxy things.
+    """Base class for Sphinxy things.
 
     """
     def __init__(self, title):
@@ -1016,7 +995,7 @@ class TestExecEnvironment(_TestProcessor):
 
     def _execute(self, test, level):
         status = Coordinator.getServiceProvider("status")
-        runnable = test is None or not test.hasFailingAncestor()
+        runnable = test is not None
         if runnable:
             if test.shouldFork:
                 self._runForked(level)
@@ -1141,11 +1120,9 @@ class Executor(object):
     """Execute a test framework function in a controlled manner, any exceptions
     are caught and re-raised"""
 
-    def __init__(self, func, test, phase, item=None, timeout=None,
-                 record=None, postFunc=None, trace=None):
+    def __init__(self, func, test, phase, item=None, record=None, postFunc=None, trace=None):
         self._func = func
         self._postFunc = postFunc
-        self._timeout = timeout
         self._item = item
         self.record = record
         self.test = test or Core.NullTest()
@@ -1159,23 +1136,13 @@ class Executor(object):
         self._runner = weakref.proxy(runner)
 
     def _invoke(self):
-        self.depth = len(inspect.stack())
-        if Suites._control:
-            Suites._control.setTestStackDepth(self.depth)
-        if self._timeout is not None:
-            WatchDog.runUnderWatchDog(self._timeout, self._func)
-            if self._postFunc:
-                WatchDog.runUnderWatchDog(self._timeout, self._postFunc)
+        if self.spec is not None and not options.get_option_value("disable_run_and_trace"):
+            self.spec.runAndTrace(self._func)
         else:
-            if self.spec is not None and not options.get_option_value("disable_run_and_trace"):
-                if Suites._control:
-                    Suites._control.setTestStackDepth(self.depth + 1)
-                self.spec.runAndTrace(self._func)
-            else:
-                _cs_preamble_ = None  # TODO: Rename to _cs_test_invoker_marker_
-                self._func()
-            if self._postFunc:
-                self._postFunc()
+            _cs_test_invoker_marker_ = None
+            self._func()
+        if self._postFunc:
+            self._postFunc()
 
     def _handleFailure(self, phaseRecord, msg=None, args=None):
         if self.test.isBug and self._phase == "run":
@@ -1224,11 +1191,11 @@ class Executor(object):
             testKillers = (Exception,)
         try:
             if self._func is not None:
-                _cs_preamble_ = None
+                _cs_test_invoker_marker_ = None
                 if self._runner:
-                    ret = self._runner.callFunc(id(self), "_invoke")
+                    _ = self._runner.callFunc(id(self), "_invoke")
                 else:
-                    ret = self._invoke()
+                    self._invoke()
             else:
                 print("Do Nowt!!!!")
         except testKillers as exc:
@@ -1271,30 +1238,26 @@ class Executor(object):
             phaseRecord.state = Core.State.EXIT_ALL
 
         # TODO: Should rename Problem -> AssertionFailure
-        except Errors.Problem as origExc:
-            self._handleFailure(phaseRecord) # DEL
+        except Errors.Problem:
+            self._handleFailure(phaseRecord)  # DEL
 
         except KeyboardInterrupt as origExc:
             msg = "Interrupted by user: %s" % origExc
-            self._handleFailure(phaseRecord, msg) # DEL
-
-        except WatchDog.Timeout as origExc:
-            msg = "Test timed out: %s" % origExc
-            self._handleFailure(phaseRecord, msg) # DEL
+            self._handleFailure(phaseRecord, msg)  # DEL
 
         except SystemExit as origExc:
             if origExc.args:
                 msg = "Unexpected sys.exit(%r)" % self.origExc.args[0]
             else:
                 msg = "Unexpected sys.exit()"
-            self._handleFailure(phaseRecord, msg) # DEL
+            self._handleFailure(phaseRecord, msg)  # DEL
 
         except Exception as origExc:
             msg = "Unhandled exception occurred:\n  %s:%s" % (
                     origExc.__class__.__name__, self.origExc)
-            self._handleFailure(phaseRecord, msg) # DEL
+            self._handleFailure(phaseRecord, msg)  # DEL
 
-        return phaseRecord # DEL
+        return phaseRecord  # DEL
 
 
 class ContinueExecutor(Executor):
