@@ -28,7 +28,7 @@ from CleverSheep.Test.Tester import Coordinator
 from CleverSheep.Test.Tester import Errors
 from CleverSheep.Test.Tester import log
 from CleverSheep.Test.Tester import options as cmdLineOptions
-from CleverSheep.Prog.Aspects import decorator, intelliprop
+from CleverSheep.Prog.Aspects import intelliprop
 
 __all__ = ["Suite", "Controller"]
 
@@ -226,7 +226,6 @@ class Controller(object):
         self._poll = None
         self._update_tid = None
         self._end = 0
-        self._testDepth = None
 
     def _update(self):
         status = Coordinator.getServiceProvider("status")
@@ -248,9 +247,6 @@ class Controller(object):
         return
         status.setField("delay", "")
 
-    def setTestStackDepth(self, depth):
-        self._testDepth = depth
-
     @property
     def poll(self):
         """The `Test.PollMan.Poller` instance used by this controller."""
@@ -462,7 +458,6 @@ class Controller(object):
         """
         return self.poll.addRepeatingTimeout(delta, cb_func, *args, **kwargs)
 
-    # TODO: This and the PollManager funciton should take *args, **kwargs.
     def addWorkFunction(self, cb_func):
         """Add function to be invoked when some work has been done.
 
